
#include <zephyr/types.h>
#include <zephyr/kernel.h>
#include <zephyr/drivers/gpio.h>
#include "common.h"
#include <stdio.h>
#include <string.h>
#include <strings.h>
extern uint8_t CMDBUFF[];
extern uint8_t CMD[];
extern const struct device *gpio0;
extern const struct device *gpio1;
extern struct PARAMS PARMS;
extern struct k_timer msclock;
extern bool KEYPADACTIVE;
extern bool KEYPAD_TIMED_OUT;
volatile extern int MILLITIME;
extern uint8_t CMD[];
extern uint32_t KEYPAD_ERR_CT;                                                 // number of successive keypad errors
extern uint32_t KEYPAD_REACTIVATE;                                             // tie to reactiveate the keypad after ERR_CT was reached
extern uint32_t KEYPAD_IDLE_LIMIT;                                             // Used to deterine keypad idle timeout
extern bool LOGGING;
extern struct gpio_callback keypad_cb_data;
uint8_t diagMsg[100];
extern struct k_fifo fifo_cmd_data; 



uint8_t* keypad_scan_once()
{       uint_least8_t COLS[3] = { COL1, COL2, COL3 };
	uint_least8_t ROWS[4] = { ROW1, ROW2, ROW3, ROW4 };
     
	static uint8_t keyState[13];                                              //  this array will contain the state of the keypad after a single scan
	uint32_t i, j, res;

	memset(keyState, '0', sizeof(keyState));                                  // clear the keypad state array

	for (i = 0; i < 3; i++)                                                   // for each column
	{
		gpio_pin_set(gpio0, COLS[i], 1);                                      // bring up the voltage on that column onky
                if(i==0)
                  j=2;
                if(i==1)
                   j=0;
                if(i==2)
                   j=1;
                gpio_pin_set(gpio0, COLS[j], 0);
		for (j = 0; j < 4; j++)                                              // for each row
		{    if(j<3)
	            res = gpio_pin_get(gpio0,ROWS[j]);                           // read the gpio port
			 else res=gpio_pin_get(gpio1,ROWS[j]);
		 if (res > 0)                                                        // if it's high
			keyState[i * 4 + j] = '1';                                       //     that column/row combination is pressed
		}
	
	}
	return (keyState);                                                        // when we're done we return the keypad state to the calling routine
}
char* keypadgetchar()
{
	uint8_t prevState[13], keysPressed[13], keyChars[] = { '1', '2', '3', '*',
			'4', '5', '6', '0', '7', '8', '9', ',' };

	static uint8_t scanResult[14], *ptr;
	uint8_t *keyState;
	uint32_t i,j,elapsed;
	uint32_t keyStart[13];                  		                         // keyStart is the time from msTimer
										                                     //   when the scan first saw the key pressed
	bool keepscanning;
	memset(scanResult, 0, sizeof(scanResult));                               // clear the scan result
	keyState=keypad_scan_once();                                             // do one scan of the keypad
	memcpy(prevState, keyState, sizeof(prevState));		                     // and save the key state
	memset(prevState, '0', sizeof(prevState));                               // clear the previous keypad state to zeroes
	memset(keysPressed, '0', sizeof(keysPressed));	                         // clear the results to zeroes
	memset(keyStart, 0, sizeof(keyStart));                                   // clear the timer array
    i=MILLITIME;
                                                                             //   and turn off timed out flag
	while (1)								                                 // now loop until there are no keys depressed
	{ 
		if(MILLITIME>KEYPAD_IDLE_LIMIT)                                      // if we've rewached the idle timeout
		{
		     KEYPAD_TIMED_OUT=true;                                          // set the timed out flag
			 return "TO";                                                    // return something unique
		}
		keepscanning = false;						                         // assume there are no keys pressed
	
		for (i = 0; i < 12; i++)					                         // compare the current key states to the previous key states
		{
			if (keyState[i] == '1')                                          // if there is a key pressed
				keepscanning = true;				                         //       set to keep scanning
			if (keyState[i] != prevState[i]) 		                         // see if there has been a state change on this key
			{
				if (keyState[i] == '1')         	                         //     if this is a new keypress
					keyStart[i] = MILLITIME;      	                         //           save the start time
				else
				{
					
                    elapsed = MILLITIME-keyStart[i];                         // otherwise see how long the key was depressed
					if (elapsed > 10) 			                             //   if the keypress lasted more than 20ms
						keysPressed[i] = '1'; 		                         //      record as a true keypress.
					keyStart[i] = 0;        		                         // in any case, clear the start time
                                                                             //  keepscanning=false;
				}
			}
		}

		if (keepscanning == false) 					                         //  if there wasn't any key pressed during his cycle
			break;							                                 //       we're done

		memcpy(prevState, keyState, sizeof(prevState));		                 // otherwise, save the key state
		keyState = keypad_scan_once();                                       // and go scan the keypad

		
	}

	memset(scanResult, 0, sizeof(scanResult)); 				                 //.now translate the keys pressed array into a string
	ptr = scanResult;
        j=0;                                                                 // check for multile keys
        for(i=0;i<12;i++)
           if(keysPressed[i]=='1')
               j++;
        if(j>1)                                                               // if this was a multi-press
            *ptr++='@';                                                       //     bracket it with 'at' signs
	for (i = 0; i < 12; i++)
	{
		if (keysPressed[i] == '1')
			*ptr++ = keyChars[i];
	}
        if(j>1)                                                                // if this was a multi-press
            *ptr++='@';  
    

	return (scanResult);                          			                    // and return it to the caller
}

void getkeypadinput()
{static uint8_t keypadinput[20],keychars[14],buffer[30];
 uint32_t i,j;
 static struct cmd_data_t *buf;
 keypadsetup();                                                                    // go setup the keypad scanner
 //vdd2_up();                                                                        // go bring up VDD2, the ILLUINATION leds and the peripherals
 memset(keypadinput,0,sizeof(keypadinput));                                        // clear the buffer
 KEYPAD_IDLE_LIMIT=MILLITIME+1000*PARMS.INATMO;                                    // set the keypad idle timeout
 KEYPAD_TIMED_OUT=false;     
 while(true)                                                                       // now scan characters in from the keypad
     {
	  strcpy(keychars,keypadgetchar());                                            // go get characters frpm the keypad
	  if(KEYPAD_TIMED_OUT)                                                         // if the keypad timed out
	      break;                                                                   // get out

      i=strlen(keychars);
      if(i==0)
          continue;
	  KEYPAD_IDLE_LIMIT=MILLITIME+1000*PARMS.INATMO;                               // reset the keypad idle timeout
	
      j=strlen(keypadinput);                                                       // see if we can fit whay we got into the buffer
      if((i+j)>sizeof(keypadinput))                                                // if we can't
         {strcpy(keypadinput,"999999999#");                                        //      put a weird value into keyad input
          break;                                                                   // and leave
         }
      strcat(keypadinput,keychars);                                                // otherwise append the new keys to what we've already stored
      i=strlen(keypadinput);                                                       // check to see if we got a '#' -the terminating character
      if(keypadinput[i-1]==RS)                                                     // if we did
           break;
     } 
 memset(CMD,0,50);
 j=strlen(keypadinput);
 if(j>10)
    {keypadinput[9]=',';
	 keypadinput[10]=0;
	}
 setup_keyscan();                                                                   // setup for the next scan
                                                          // stop the milisecond timer   
 if(KEYPAD_TIMED_OUT)                                                               //  if the keypad tied out
 {                 
	  vdd2_up();                                                                    // bring up VDD2 (including I2C)
	  if(!LOGGING)
	      logmsg_add(2, "KEYPAD TIME OUT");                                         // log a diagnostic record
	  vdd2_down();                                                                  // take dwn VDD2
	  return;                                                                       //     and   we're done      
 }                                              
 memset(buffer,0,sizeof(buffer));                                                   // build the command - cler a work area
 strcpy(buffer,"XMD");                                                              // Put in the "XMD" string
 strcat(buffer,keypadinput);                                                        // append the keypad input up vdd2 fully(VDD2 is already set on, as are the ILLUMINTION LEDs)
 buf=k_malloc(sizeof(*buf));                                                        // allocate memory for the fifo data
 vdd2_up();                                                                         // bring
 if(buf==NULL)
    {
       if(!LOGGING)
	      logmsg_add(2, "KEPPAD KMALLOC FAILED");

	}
	else 
	{
      memcpy(buf->data,buffer,strlen(buffer));             // copy in the nexly built command
      buf->len=strlen(buffer);                             //      and the length of the command
      k_fifo_put(&fifo_cmd_data,buf);                      // and end it off to proccmd
     
	}
 
                                                       
 
 return;

 }





