#include <zephyr/kernel.h>
#include <zephyr/types.h>
#include  <stdio.h>
#include <string.h>
#include <zephyr/device.h>
#include <soc.h>
#include "common.h"
#include <zephyr/drivers/gpio.h>

#include <zephyr/settings/settings.h>

#include <zephyr/pm/pm.h>
#include <zephyr/pm/device.h>
#include <zephyr/pm/policy.h>
#define STACKSIZE 4096
#define PRIORITY 7
extern const struct device *gpio0;  
extern const struct device *gpio1;
extern const struct device *uart0;
extern struct EEPROMMAP MAP;
extern struct k_fifo fifo_uart_tx_data;
extern struct k_fifo fifo_uart_rx_data;
extern struct k_sem ble_init_ok;
extern struct k_timer alarm;
extern struct k_timer msclock;
extern struct k_sem voltage_mon_on;
extern struct k_sem voltage_mon_off;
extern uint16_t LOW_VOLTAGE;                                                           // the lowest battery voltage seen while the motor is spinning
extern uint32_t LVCNT;
extern struct PARAMS PARMS;
volatile extern bool ALARM;
extern bool LOGGING;
extern bool QPOWEREDUP;
extern bool QCONNECTED;
extern bool VDD2_UP;
volatile extern bool NO_BT_DISP;

extern uint32_t MILLITIME;
extern bool TIMER_RUNNING;
volatile bool PATTERNFOUND;
extern uint8_t PATTERN[];
extern uint8_t PATTERNREC[];
extern uint8_t PREVREC[];
struct QMSG {                                                             // quectel responses are stored in a list
	         struct QMSG *next;                                           // pointer to the next entry in the list
             uint8_t len;                                                 // the length of this entry
			 uint8_t data[];                                              // the message content                                            
            };
extern struct k_mutex quelock;
volatile uint32_t QMSGCNT=0;  
                                                                          // the number of messages currently stored in qmsgs

                                                                          // a single uint8_t length of the message
																		  // the contents of the message in uint8_t 
extern struct k_fifo fifo_uart_tx_data;
extern struct k_fifo fifo_uart_rx_data;




uint32_t quectel_powerup()                                                // power up the Quectel chip
{
	uint32_t i,ret, ret1, ret2, ret3, ret4, ret5,start;
	uint8_t buffer[30];
	QCONNECTED=false;

	TIMER_RUNNING=true;
	MILLITIME=0;                                                         // reset the millisecond timer
    k_timer_start(&msclock, K_MSEC(1), K_MSEC(1));                       // and start it up 
    start=MILLITIME;
	
    ret = gpio_pin_configure                                              // configure Q_STATUS as PULL_UP (cures one of the volatge leaks on VDD3)
	      (gpio0, Q_STATUS, GPIO_INPUT |  GPIO_PULL_UP | GPIO_ACTIVE_HIGH);

   
	sprintf(buffer,"TOP OF PWRUP - %d mS", MILLITIME);                     // and report it
	bt_send_data(buffer);   
	gpio_pin_set(gpio1,Q_PWRKEY,1);                                       // insure that PWRKEY is low (The signal is iinverted)
    k_sleep(K_MSEC(50));                                                  // sleep for 50 mS  
	gpio_pin_set(gpio0,VDD3_ENABLE,1);                                    // bring up VDD3 
	k_sleep(K_MSEC(100));                                                 // sleep for 100 msec
	pm_device_action_run(uart0, PM_DEVICE_ACTION_RESUME);                 // start the UART
	sprintf(buffer,"VDD3 UP - %d mS", MILLITIME);                         // and report it
	bt_send_data(buffer);   
	k_sleep(K_MSEC(80));                                                  // sleep for 80 mS 
 
	ret = gpio_pin_get(gpio0,Q_STATUS);                                   // debugging - check q_status
	sprintf(buffer,"PWRUP  2- STATUS %d", ret);                           // and report it
	bt_send_data(buffer); 
	
	gpio_pin_set(gpio1,Q_PWRKEY,0);                                       // bring up PWRKEY 
	sprintf(buffer,"PWRKEY UP - %d mS", MILLITIME);                       // and report it
	bt_send_data(buffer);    
    k_sleep(K_MSEC(50));                                                  // leave it up for 50 mS
	gpio_pin_set(gpio1,Q_PWRKEY,1);                                       // and pull it down again
	sprintf(buffer,"PWRKEY DOWN - %d mS", MILLITIME);                     // and report it
	bt_send_data(buffer);    
	k_sleep(K_MSEC(500));                                                 // sleep for 750 mS
	gpio_pin_set(gpio1,Q_PWRKEY,0);                                       // and pull it back up
	sprintf(buffer,"PWRKEY UP - %d mS", MILLITIME);                       // and report it
	bt_send_data(buffer);    
	for(i=0;i<40;i++)                                                     // mow - loop waiting for status to come high
	{
		ret = gpio_pin_get(gpio0,Q_STATUS);                               // check q_status
		if(ret==0)                                                        // if status is high (the signal is inverted)
		{
			printf(buffer,"STATUS HGH AT %d mS",i*100);                  // send a diag message
			bt_send_data(buffer);                                        // send a diag message
			PATTERNFOUND=false;                                           //  now - st upp to wait for the 'AP RDY' from the quectel chip
			ALARM=false;                                                  // reset the ALARM logical
	        strcpy(PATTERN,"APP RDY");                                    // tell the message received thread what to look for
 	        k_timer_start(&alarm,K_MSEC(5000),K_NO_WAIT);                 // set the alarm clock to 5 seconds
	        while((!ALARM)&&(!PATTERNFOUND))                              // now wait for either "AP RDY" "to be recevied or the alarm to go off
	        {
                k_sleep(K_MSEC(50)); 
		    }
			k_timer_stop(&alarm);
			break;                                                        // when we get out - leave the for loop

		}
		k_sleep(K_MSEC(100));                                             // othwewisde sleep for 100 mS
	}
	k_timer_stop(&msclock);  
	TIMER_RUNNING=false;

	if((ret==1)||(ALARM))                                                 // if we didn't powerup or 'AP RDY'
	{
		sprintf(buffer,"FAILED at %d mSEC",MILLITIME);
                                     // loop waiting on bt_send
		bt_send_data(buffer);
		sprintf(buffer,"PWRUP FAIL AT %d", MILLITIME);                     // builld an error message
		 if(!LOGGING)
            logmsg_add(2,buffer);                                             // and log it
		QPOWEREDUP=false;                                                 // set the logical
	    pm_device_action_run(uart0, PM_DEVICE_ACTION_SUSPEND);            // stop the UART
		gpio_pin_set(gpio0,VDD3_ENABLE,0);                                // bring down VDD3 
		gpio_pin_configure(gpio0, Q_STATUS,GPIO_DISCONNECTED);            // deconfigure Q_STATUS
		return 1;                                                         // return failure
	}
	QPOWEREDUP=true;
	return 0;

}
uint32_t quectel_powerdown()                                              // power down the Quectel chip
{
	uint32_t ret,i;
	bool quecup;
	quecup=false;
	uint8_t buffer[30];

	
	if(!QPOWEREDUP)                                                       // if the chip isn't powered up
	{

	    ret = gpio_pin_configure                                          //        de-configure Q_STATUS 
	      (gpio0, Q_STATUS, GPIO_DISCONNECTED);                            
	    gpio_pin_set(gpio0,VDD3_ENABLE,0);                                //         remove power from  the Quectel chip 
		gpio_pin_set(gpio1, Q_PWRKEY,0);                                  // bring down PWRKEY 
		QPOWEREDUP =false;   
		QCONNECTED= false;
	    pm_device_action_run(uart0, PM_DEVICE_ACTION_SUSPEND);            // stop the UART
		gpio_pin_set(gpio0,VDD3_ENABLE,0);                                // bring down VDD3 (note that it may not be up)
        return 1;                                                         //         and return failuer
	}
    gpio_pin_set(gpio1, Q_PWRKEY,1);                                      // bring down PWRKEY 
	k_sleep(K_MSEC(750));                                                 // sleep 750 milliseconds
	gpio_pin_set(gpio1,Q_PWRKEY,0);                                       // now bring PWRKEY back up- the chip should be powering down
	ALARM=false;
	PATTERNFOUND=false;                                                   //  now - sEt up to wait for the 'NORMAL POWER DOWN' from the quectel chip
	strcpy(PATTERN,"NORMAL POWER DOWN");                                  // tell the message received thread what to look for
    k_timer_start(&alarm,K_MSEC(5000),K_NO_WAIT);                         // e'll wait for five seconds
	while((!ALARM)&&(!PATTERNFOUND))                                      // now wait for either "NORMAL POWER DOWN" "to be recevied or the alarm to go off
	{
        k_sleep(K_MSEC(50)); 
	 }
	k_timer_stop(&alarm);
	if(!ALARM)                                                            // if we saw the pattern
	{                                                                     // 20 loops is 2 seconds

	    ret = gpio_pin_configure                                          //  diconnect Q_STATUS
		      (gpio0, Q_STATUS, GPIO_DISCONNECTED);  
		pm_device_action_run(uart0, PM_DEVICE_ACTION_SUSPEND);            // stop the UART
	    gpio_pin_set(gpio0,VDD3_ENABLE,0);                                //  remove power from  the Quectel chip 
		gpio_pin_set(gpio1, Q_PWRKEY,0);                                  // bring down PWRKEY
		QPOWEREDUP =false; 
		QCONNECTED=false;
        return 0;                                                         // and return success		
	}
	                                                                      // if we get hre - the chip didn't power down gracefully
	ret = gpio_pin_configure                                              //        disconnect Q_STATUS
	     (gpio0, Q_STATUS, GPIO_DISCONNECTED);
    pm_device_action_run(uart0, PM_DEVICE_ACTION_SUSPEND);                // stop the UART
	gpio_pin_set(gpio0,VDD3_ENABLE,0);                                    //         remove power from  the Quectel chip  
	gpio_pin_set(gpio1, Q_PWRKEY,0);                                      // bring down PWRKEY  
	
	QPOWEREDUP=false;
	QCONNECTED=false;
	return 1;                                                             //         and ret_power_uourn failure
}

void qmsgsend(uint8_t *command)                                           // send a command to the quectel module
{
 uint8_t cmdout[200],*ptr,buffer[200];
 
 uint32_t i;
                                                           // clear out any messages not processed
 QMSGCNT=0;                                                              // set the received messages count to 0
 //k_mutex_lock(&quelock,K_FOREVER);                                       // set the mutex
 memset(cmdout,0,sizeof(cmdout));                                        // clear the commandout buffer to nulls
 strcpy(cmdout,command);                                                 // retrieve the comman
 i=strlen(cmdout)-1;                                                     // point to the end of the command
 if(cmdout[i]!='\r')                                                     // if it isnt terminated with a  carriage return
    cmdout[i+1]='\r';                                                    // supply one

 uart_send(cmdout);                                                      // and send the command to the uart


ptr=cmdout;
 while(*ptr!='\r')
 {
	if(*ptr==',')
	   *ptr=';';
	ptr++;
 }
 
 sprintf(buffer," SENT %d",strlen(cmdout));
 i=strlen(cmdout);
 if(i>60)
     cmdout[60]=0;
 
 *ptr=0;
 strcat(cmdout,buffer);
  if(!NO_BT_DISP)
       bt_send_data(cmdout);
 //k_mutex_unlock(&quelock);

}
void qmsgrecv()
{
 struct uart_data_t *uart_buf;
 struct QMSG  *newmsg;
 uint32_t ret,i,j;
 uint8_t buffer[1100],buffer2[200],buffer3[100],*ptr, *ptr2;
 k_sem_take(&ble_init_ok, K_FOREVER);
 bool fst1062=false;
 bool fst1063 = false;                                    // wait until the program has initilaized
 while(true)                                                             // - now - retrieve the responses
   {

      uart_buf=k_fifo_get(&fifo_uart_rx_data,K_MSEC(100));              // wait for a reply - note the 	1000 msec 
	                                                                    // is totally arbitrary 
      if(uart_buf==NULL)                                                 // if nothing came back (note that the read waits up to timeout ms for data tro come back)                                             
		   continue;                                                     //     loop
	  QMSGCNT++;                                
	  ptr=uart_buf->data;;                                               // parse the message
	  ptr2=uart_buf->data;	  
	  i=strlen(PATTERN);                                                 //   looking for a pattern to match
	  while((ptr-ptr2) < uart_buf->len)                                 // this cleans up message displays later on
	  {
	   ret=strncmp(PATTERN,ptr,i);                                       // look for a pattern match;
	   if(ret==0)                                                        // if we found the pattern
	   {
	        PATTERNFOUND=true;                                            //      let the world know
			memset(PATTERNREC,0,1100);                                    // clear the save record
			memmove(PATTERNREC,uart_buf->data,uart_buf->len);             // and save what we got fro the uart
	   }
       if(*ptr==',')
	      *ptr=';';
	   ptr++;
	  } 
	  if((uart_buf->len==1078)&&(!fst1062))                                 // if this is the first 550 byte record
	  {
		memmove(buffer,uart_buf->data,uart_buf->len); 
		fst1062=true;
		for(i=0;i<22;i++)                                                // lisyt off the contents 50bytes at a time
		{
			memset(buffer3,0,sizeof(buffer3));
			memmove(buffer3,&buffer[i*50],50);
			for(j=0;j<50;j++)
			   if(buffer3[j]==',')
			       buffer3[j]=';';
			   
			bt_send_data(buffer3);
		}

	  }
	  if((uart_buf->len==1079)&&(!fst1063))                                 // if this is the first 550 byte record
	  { 
		memmove(buffer,uart_buf->data,uart_buf->len); 
		fst1063=true;
		for(i=0;i<22;i++)                                                // lisyt off the contents 50bytes at a time
		{
			memset(buffer3,0,sizeof(buffer3));
			memmove(buffer3,&buffer[i*50],50);
			for(j=0;j<50;j++)
			if(buffer3[j]==',')
				buffer3[j]=';';
			bt_send_data(buffer3);
		}
		memset(buffer3,0,sizeof(buffer3));
		memmove(buffer3,&PATTERNREC[550],1);
		bt_send_data(buffer3);

	  }
	  memset(PREVREC,0,100);                                              // we save every record. This is just to accommdate the IMEILD command
	  memmove(PREVREC,buffer,30);                                         // which needs the IMEI numer -- displayed just before the pattern
	  memset(buffer,0,sizeof(buffer));                                   // clear them temoorary send buffer
	  memmove(buffer,uart_buf->data,uart_buf->len);                      // load the quectel response inti the buffet
	  if(buffer[strlen(buffer)-1]=='\r')                                 // if there is a terminiating return
	     buffer[strlen(buffer)-1]=0;                                     // eliminate it
	  if(strlen(buffer)>50)                                              // if the buffer is longer than 60 characters
	     buffer[50]=0;                                                   // truncate it
	  sprintf(buffer2, " RECEIVED %d",uart_buf->len);
	  strcat(buffer,buffer2);
	  for(i=0;i<strlen(buffer);i++)
	     {
			if ((buffer[i]<32)||(buffer[i]>126))
			  if((buffer[i]!=10)&&(buffer[i]!=13))
			   buffer[i]='!';
			if(buffer[i]==',')
			     buffer[i]=';';
		 }
	  bt_send_data(buffer);                                               // and send it out

	  k_free(uart_buf);
	  

	//ret=k_mutex_unlock(&quelock);
	//if(ret!=0)
	//    bt_send_data("MUTEX ERR 4");
	
    continue;
   }
}
K_THREAD_DEFINE(qmsgrecv_id, STACKSIZE, qmsgrecv, NULL, NULL,
		NULL, PRIORITY, 0, 0);

void qcertload(uint32_t certnbr)                                         // load a single certificate
{
	uint8_t *certnames[]   = {"cacert",
	                          "clicert",
							  "clikey"
							  };
    uint8_t command[100], buffer[128],buffer2[100],*ptr,cksum1[6], cksum2[6];
	struct QMSG *msgptr;
    uint32_t i,j;
	uint32_t address, lth;
	address=MAP.addr[certnbr*2+1];                                      // get the base address of the certificate
	lth=MAP.addr[certnbr*2+2];                                          // and its length
	sprintf(command,"AT+QFUPL=\"%s\",%d,%d\r",certnames[certnbr],lth,100);// build the upload command
	ALARM=false;                                                        // reset the ALARM flag
	PATTERNFOUND=false;                                                 //   and the PATTERNFOUND flag
	strcpy(PATTERN,"CONNECT");                                          // tell the message received thread what to look for
	k_timer_start(&alarm,K_MSEC(30000),K_NO_WAIT);                      // set the alarm clock to 30 seconds
	qmsgsend(command);                                                  // send the comand to the Quectel module   
	while((!ALARM)&&(!PATTERNFOUND))                                    // now wait for either the pattern to be rveived or the alar to go off
	{
        k_sleep(K_MSEC(50));
	}                                                                   // wait for the "CONNECT"
    if(ALARM)  
	{                                                                   // if we didn't
		k_timer_stop(&alarm);                                           // stop the timer
		bt_send_data("ALARM EXPIRED");                                  // inform the user
		QCONNECTED=false;                                               // and flag the connection as failed
		sprintf(buffer,"QCRTLD FAILED AT %d", certnbr);                 // build an error message
		 if(!LOGGING)
           logmsg_add(2,buffer);                                            // and log it
		bt_send_data(buffer);                                            // display it
		return;                                                          // and get out
	}   
	k_timer_stop(&alarm);                                             //  otherise, stop the timer
	ALARM=false;                                                      // reset the ALARM flag
	PATTERNFOUND=false;                                               //   and the PATTERNFOUND flag
	strcpy(PATTERN,"+QFUPL");                                         // tell the message received thread what to look for
	k_timer_start(&alarm,K_MSEC(30000),K_NO_WAIT);                    // set the alarm clock to 30 seconds
	sprintf(buffer,"I*=%d",i);                                        // note that we setthis up BEFORE we send the cert data
	                                                                  // to the quectel chip
	bt_send_data(buffer);
    bt_send_data("SENDING CERT");
    while(lth>0)                                                        // now - send to data to the quectel chip
	{
     memset(buffer,0,sizeof(buffer));                                   // clear the buffer to nulls
	 if(lth>32)                                                        // figure out how many bytes to get
	     i=32;
	 else i=lth;
	 eeprom_retrieve(address,i,buffer);                                 // retrieve the bytes
	                                            // send them over to the quectel chip
	 j=strlen(buffer);

	 sprintf(buffer2," SENT %d",i);
	 bt_send_data(buffer2);
	 uart_send(buffer);      
	
	
	
	 address+=i;                                                        // move to the next piece of cert
	 
	 lth-=i;                                                            // reduce the length to be sent

	}

	 

	 while((!ALARM)&&(!PATTERNFOUND))                                  // now wait for either the pattern to be rveived or the alar to go off
	     {
             k_sleep(K_MSEC(50));
		 }
	 if(ALARM) 
	 {                                                                 // if it went off
		    k_timer_stop(&alarm);                                      // stop the timer
			bt_send_data("ALARM EXPIRED");                             // inform the user
			QCONNECTED=false;                                          // and flag the connection as failed
			sprintf(buffer,"QCRTLD FAILED AT %d", certnbr);            // build an error message
			 if(!LOGGING)
                logmsg_add(2,buffer);                                      // and log it
			bt_send_data(buffer);                                      // display it
			return;                                                    // no get out
	
	 }
	k_timer_stop(&alarm);   
	bt_send_data("PASSED");
	return;


	

}
          



void qcertsload()                                                        // load the certs into quectel ROM
{   
	qmsgsend("AT+QFLST");                                                // go see it there are alreaddy certs loaded
	k_sleep(K_MSEC(100));                                                // tzkd  little nap
  	if(QMSGCNT>4)                                                       // if there are (msgcnt > 3 )
	     {qmsgsend("AT+QFDEL=\"*\"\r");                                     //     delete them

		 }
	qcertload(0);
	qcertload(1);
	qcertload(2);
	bt_send_data("CERTS LOADED");
	
}

int32_t qconnect()                                                         // conect to AWS
{   
	uint32_t i,j,msgschecked;
	
	uint32_t address, lth;
	uint8_t buffer[100], endpoint[50],deviceid[15],QMTOPEN[80],QMTCONN[80],QMTSUB[80];
	struct QMSG *msg; 
	bool okfound;
	struct qcmd {uint8_t command[80];
                 uint8_t response[50];
				 uint32_t timeout;
				} qcmds[] = {
					    //   {"AT+QIDEACT","OK",300},
						//	 {"AT+cfun=0","OK",4000},
                        //   {"AT+CGDCONT=1,\"ip\",\"iot.aer.net\"\r","OK",4000},
                        //   {"AT+cfun=1","OK",4000},
				        //   {"AT+QCFG=\"iotopmode\",2\r","OK",10000},                            // entry 0 (T-Mobile)
						     {"AT+QCFG=\"iotopmode\",2\r","OK",10000},                            // entry 0 (AERIS)
						//	 {"AT+QCFG=\"band\",8,81A,1A\r","OK",4000},                           //       1 (T-MOBILE)
							 {"AT+QCFG=\"band\",1081A,1081A,1081A\r","OK",4000},                  //       1 (AERIS)
							 {"AT+CEREG?;+QNWINFO;+QCSQ\r","OK",4000},                            //       2
						//	 {"AT+QICSGP=2,1,\"iot.aer.net\",\"\",\"\",0\r","OK",4000},           //       3 (AERIS)
						//	 {"AT+QICSGP=2,1,\"iot.catm\",\"\",\"\",0\r","OK",4000},              //       3 (T-Mobile)
						     {"AT+CGDCONT=1,\"ip\",\"iot.aer.net\"\r","OK",10000},                // entry 3 (AERIS)
							 {"AT+QMTCFG=\"ssl\",2,1,2\r","OK",4000},                             //       4
							 {"AT+QMTCFG=\"version\",2,4","OK",4000},                             //       5
							 {"AT+QSSLCFG=\"cacert\",2,\"cacert\"\r","OK",4000},                  //       6
							 {"AT+QSSLCFG=\"clientcert\",2,\"clicert\"\r","OK",4000},             //       7
                             {"AT+QSSLCFG=\"clientkey\",2,\"clikey\"\r","OK",4000},               //       8
							 {"AT+QSSLCFG=\"seclevel\",2,2\r","OK",4000},                         //       9
							 {"AT+QSSLCFG=\"sslversion\",2,4\r","OK",4000},                       //      10
							 {"AT+QSSLCFG=\"ciphersuite\",2,0XFFFF\r","OK",4000},                 //      11
							 {"AT+QSSLCFG=\"ignorelocaltime\",2,1\r","OK",4000},                  //      12
							 {"","+QMTOPEN:",30000},                                              //      13
							 {"","+QMTCONN:",30000},                                              //      14
						     {"","+QMTSUB:",30000},                                               //      15
							 {"END","END",4000}
				
							};
  
    memset(QMTOPEN,0,sizeof(QMTOPEN));                                   // clear the three variable commands to NULLS
	memset(QMTCONN,0,sizeof(QMTCONN));
	memset(QMTSUB,0,sizeof(QMTSUB));
	k_sleep(K_MSEC(1000));                                              // build the QMTOPEN command
    address=MAP.addr[7];                                                // get the base address of the endpoint
	lth=MAP.addr[8];                                                    // and it's length  
	if(!VDD2_UP)                                                        // if vdd2 isn=t uo
	       vdd2_up();                                                   // bring up vdd2
	memset(endpoint,0,sizeof(endpoint));                                // clear the endpout to NULLS
    eeprom_retrieve(address,lth,endpoint);                              // go rettrieve the endpoint
	//strcpy(endpoint,"a10arjmfohzbr8-ats.iot.us-east-1.amazonaws.com");
	sprintf(QMTOPEN,"AT+QMTOPEN=2,\"%s\",8883\r",endpoint);               // and format and save the command
	memset(deviceid,0,sizeof(deviceid));                                // build the QMTCONN command
	memmove(deviceid,PARMS.lockid,sizeof(PARMS.lockid));                // get the lock id
	sprintf(QMTCONN,"AT+QMTCONN=2,\"%s\"\r",deviceid);                    // format and save the QMTCONN message
	strcpy(QMTSUB,"AT+QMTSUB=2,1,\"/");                                    // build the QMTSUB command
	strcat(QMTSUB,deviceid);                                            // add the deviceid                                             // and the balance of the command
	strcat(QMTSUB,"/confirm\",1");
	strcpy(qcmds[13].command,QMTOPEN);                                  // load the new commands into the array
    strcpy(qcmds[14].command,QMTCONN);  
    strcpy(qcmds[15].command,QMTSUB);
	QCONNECTED=true;                                                    // assume that the connection will work
    i=0;
	k_sem_give(&voltage_mon_on);                                        // start voltge monitoring  
	while(true)                                                         // now -- excecute the  commands
	{
     j=strcmp(qcmds[i].command,"END");
	 if(j==0)
	      break;
	 ALARM=false;                                                      // reset the ALARM flag
	 PATTERNFOUND=false;                                               //   and the PATTERNFOUND flag
	 strcpy(PATTERN,qcmds[i].response);                                // tell the message received thread what to look for
	 k_timer_start(&alarm,K_MSEC(qcmds[i].timeout),K_NO_WAIT);         // set the alarm clock to max wait
	 qmsgsend(qcmds[i].command);                                       // and send the command to the Quectel module
	 while((!ALARM)&&(!PATTERNFOUND))                                  // now wait for either the pattern to be rveived or the alar to go off
	     {
             k_sleep(K_MSEC(50));
		 }
	 if(ALARM) 
	 {                                                                 // if it went off
		    k_timer_stop(&alarm);                                      // stop the timer
			bt_send_data("ALARM EXPIRED");                             // inform the user
			QCONNECTED=false;                                          // and flag the connection as failed
			sprintf(buffer,"QCONN FAIL AT %d", i);                     // builld an error message
			 if(!LOGGING)
                logmsg_add(2,buffer);                                      // and log it
			k_sem_give(&voltage_mon_off);                              // stop monitoring the voltage
            sprintf(buffer,"LOW VOLTAGE %d CNT %d",LOW_VOLTAGE, LVCNT);                  
             bt_send_data(buffer);
			return;                                                    // no get out
	
	 }
	k_timer_stop(&alarm);                                             // in any case - stop the timer
	 i++;                                                              // and move on to the next timer
	 if(i==1)
	    i=2;

	}
	k_sem_give(&voltage_mon_off);                                      // stop monitoring the voltage
    sprintf(buffer,"LOW VOLTAGE %d CNT %d",LOW_VOLTAGE, LVCNT);        // and report the results              
    bt_send_data(buffer);
    bt_send_data("OK");

}
 