/*
 * Copyright (c) 2017 comsuisse AG
 *
 * SPDX-License-Identifier: Apache-2.0
 */


#include <zephyr/kernel.h>
#include  <stdio.h>
#include <string.h>
#include <zephyr/drivers/gpio.h>
#include <zephyr/drivers/i2c.h>
#include <zephyr/sys/printk.h>
#include <zephyr/sys/reboot.h>
#include "common.h"


// PCF8523 REGISTERS
// control and status registers
#define RTC_CONTROL_1_REG       0x00
#define RTC_CONTROL_2_REG       0x01
#define RTC_CONTROL_3_REG       0x02
// counters for the clock function
#define RTC_SECONDS_REG         0x03
#define RTC_MINUTES_REG		0x04
#define RTC_ HOURS_REG		0x05
#define RTC_DAYS_REG		0x06
#define RTC_WEEKDAYS_REG        0x07
#define RTC_MONTHS_REG		0x08
#define RTC_YEARS_REG		0x09
// alarm condition 
#define RTC_MINUTE_ALARM_REG    0x0A
#define RTC_HOUR_ALARM_REG      0x0B
#define RTC_DAY_ALARM_REG       0x0C
#define RTC_WEEKDAY_ALARM_REG   0x0D
//offset calibration 
#define RTC_OFFSET_REG		0x0E
//timers 
#define RTC_CLKOUT_CTRL_REG     0x0F
#define RTC_A_FREG_CTRL_REG     0x10
#define RTC_A_REG		0x11
#define RTC_B_FREG_CTRL_REG     0x12
#define RTC_B_REG		0x13
// clock out settings
#define RTC_CLKOUT_1_HZ		0x30
#define RTC_CLKOUT_32_HZ        0x28
#define RTC_CLKOUT_1024_HZ      0x20
#define RTC_CLKOUT_4096_HZ      0x18
#define RTC_CLKOUT_8192_HZ      0x10
#define RTC_CLKOUT_DISABLED     0x38

extern const struct device *i2c1;
extern const struct device *gpio0; 
extern const struct device *gpio1;
extern int32_t HOURSTOREBOOT;
extern bool QPOWEREDUP;
extern bool QCONNECTED;
extern bool LOGGING;
extern uint8_t RTCRESETCODE;
extern struct PARAMS PARMS;
extern struct EEPROMMAP MAP;

uint8_t to_bcd(int16_t value)
{uint8_t work1, work2, work3;
 work1=value;
 work2=work1/10;                                                                  // dividevalue by 10 to get the high orer 4 bits
 work3=work1-work2*10;                                                            // the low orer foiur bits are the remainder
 work2=work2<<4;                                                                  // left shift the high order bits
 work1=work2+work3;                                                               // assemble the bcd vale
 return work1;                                                                    // and send it back
}
int16_t from_bcd(uint8_t value)
{uint8_t work1, work2;
 int16_t newvalue;
 work1=value&0X0F;                                                               // isolate the low order 4 bits
 work2=value>>4;                                                                  // isolate the high order 4 bits
 work2=work2*10;
 newvalue=work1+work2;
 return newvalue;
}
void calc_reboot()
{
  struct RTCDATE date;
  int32_t i; 
  uint32_t posix1,posix2,posix3;  
  uint8_t buff[50];
  date=get_rtc();                                                                // go get the rtc date
  posix1=dt_to_posix(date);                                                      // convert it to a posix date
  posix2=posix1/3600;                                                            // caculate hours since 1/1/70
  posix2=posix2*3600;                                                            // converted back to seconds
  posix3=posix1/604800;                                                          // the posix date for the last Thursday
  posix3=posix3*604800;                                                          // coverted back into seconds
  i=(posix2-posix3)/86400;                                                       // days since last thursday
  if(i<3)                                                                        // if today is Thursday, Friday or Saturday
       posix3=posix3+3*86400;                                                    // next Sunday is here
  else posix3=posix3+10*86400;                                                   // otherwise, it's here
  posix3=posix3+25200;                                                           // at 7:00 AM GMT
  i=posix3-posix2;                                                               // the number of seconds till reboot
  HOURSTOREBOOT=i/3600;                                                                      // the number of hour till reboot
  sprintf(buff,"%d HOURS UNTIL REBOOT",HOURSTOREBOOT);
  bt_send_data(buff);

}
uint32_t set_rtc_offset(int16_t xoffset)                                         // set the RTC offset
{int8_t reg,addr,offset,test,b1,b2;
 uint8_t data[2], msg[30];
 int8_t offsetval,prevoffset;

 int32_t ret,i,longoffsetval,prevoffsetval,newoffsetval;
 addr=RTC_OFFSET_REG;                                                             // set the offset register address
 i2c_write_read(i2c1,RTCADDR,&addr,1,&reg,1);                                     //    and get the current offset
 prevoffset=reg;                                                               
 test=reg&0X40;                                                                   // test bit 6
 if(test==0)                                                                      // if this bit set the current offset is positive
      reg=reg&0X7F;                                                               //     so we turn off the high order bit
 prevoffsetval=reg;                                                               // we work with 32 bit ints for a while now 
 offset=xoffset;                                                                  //  save the given offset
   
 if(xoffset==0)                                                                   // if the requestested offset is 0
     return prevoffsetval;                                                        // return the current offet
 if(prevoffsetval!=0)                                                             // if it's not 0                                             
     prevoffsetval=prevoffsetval*407/100;                                         //    convert it back to PPM
 else prevoffsetval=0;
 i=prevoffsetval+offset;                                                          // calculate the new offset value
 if((i>256)||(i<-260))                                                            // if the new offset is out of bounds
 {
      sprintf(msg,"Current offset is %d",prevoffsetval);                          // let the user know
      bt_send_data(msg);
      sprintf(msg,"Adding %d gives %d",offset,i);
      bt_send_data(msg);
      bt_send_data("Which is out of bounds");
      return 1;

 }

 longoffsetval=offset;                                                            // get the adjutment
 if(xoffset!=999)                                                                 // if the offset isn't 999
 {
     longoffsetval=longoffsetval+prevoffsetval;                                   // the new offset is a net of the old offset and the adjustemt
     newoffsetval=longoffsetval;                                                  // save this for subsequent display
     if(longoffsetval!=0)                                                         // if the new value is not 0
        longoffsetval=longoffsetval*10000/407;                                    //     convert from PPM to the RTC 4.07 PPM unit
     i=longoffsetval%100;                                                         // we're going to round the result using this
         longoffsetval=longoffsetval/100;                                         // first dividing by 100
     if(i>49)                                                                     // then testing the remainder
        longoffsetval++;                                                          // and adding 1 if appropriate  for positive numbeers
     if(i<-49)
       longoffsetval--;                                                            // or subtracting 1 for negative numbers
     offset=longoffsetval;                                                         // now - convert it back into an 8 bit integer
 }
 else {                                                                            // 999 resets the register  to 0
        offset=0;     
        newoffsetval=0;
      }                                                                           
 if(offset>=0)                                                                    // if it's positive
     offset=offset|0X80;                                                          // turn on the high order bit
 test=prevoffset&0x80;                                                            // isolate the high irder bit of the old offset
 if(test!=0)
      b1=1;
 else b1=0;
 test=offset&0x80;                                                                // isolate the high order bit of the new offset
 if(test!=0)
     b2=1;
 else b2=0;

 sprintf(msg,"OFFSET WAS %d %d-0X%02X",prevoffsetval,b1,prevoffset&0X7F);
 bt_send_data(msg);
 sprintf(msg,"OFFSET  IS %d %d-0X%02X",newoffsetval,b2,offset&0X7F);
 bt_send_data(msg);
 addr=RTC_OFFSET_REG;                                                             // go read the offset register
 ret=i2c_write_read(i2c1,RTCADDR,&addr,1,&reg,1);                                 // 
 if(ret!=0)
     {//printk("WTC\n");                                                            // if the read failed
      return ret;    
     }
 data[0]=RTC_OFFSET_REG;                                                          // now apply the offest
 data[1]=offset;                                                               // to the RTC
 i2c_write(i2c1,data,2,RTCADDR);                                              
 return 0;
}  
                                                     
void set_rtc(struct RTCDATE date)
{
 int ret,i;
 uint8_t writebuff[8], readbuff[20], test,buff[30];
 uint32_t posix;
 uint16_t wday;
 volatile struct RTCDATE defdate,xdate;
                                  // disable the RTC interrupt

 xdate=date;
 posix=dt_to_posix(xdate);                                                        // convert the date to posix
 sprintf(buff,"POSIX %d",posix);
 bt_send_data(buff);
  posix=posix/86400;                                                               // divide by 86400 - gives sdays since 1/1/70
 posix=posix%7;                                                                   // days  since Thursday
 wday=posix+4;
 if(wday>6)
        wday=wday-7;
 sprintf(buff,"WEEKDAY IS %d",wday);
 bt_send_data(buff);
 writebuff[0]=RTC_SECONDS_REG;                                                    // point at the first register
 writebuff[1]=to_bcd(date.second);                                                // now load the date into the write string
 writebuff[2]=to_bcd(date.minute);
 writebuff[3]=to_bcd(date.hour);
 writebuff[4]=to_bcd(date.day);
 writebuff[5]=to_bcd(wday);
 writebuff[6]=to_bcd(date.month);
 writebuff[7]=to_bcd(date.year-2000);
 ret=i2c_write(i2c1,writebuff,8,RTCADDR);                                          // put the new values into the clock
 if(ret!=0)                                                                       // if thre was a failure
      {                                                                            
         if(!LOGGING)
            logmsg_add(2,"RTC WRITE DATE FAILED");                                     // write an error  message to msy 0 pool
       return;                                                                    // and return
      }
  writebuff[0]=RTC_CONTROL_3_REG;                                                 // now point at CONTROL_REG_3
 ret=i2c_write_read(i2c1,RTCADDR,writebuff,1,readbuff,1);                         // and read the register
 if(ret!=0)                                                                       // if there was a failure
      {                                                                            
         if(!LOGGING)
           logmsg_add(2,"RTC GET REG 3 FAILED");                                     // write an error  message to msy 0 pool
       return;                                                                    // and return
      }
 test=readbuff[0]&0X1F;                                                           // set the three high order bits to 0
 writebuff[0]=RTC_CONTROL_3_REG;                                                  // set up control register 3
 writebuff[1]=test;                                                               // ocntents are
                                                                                  //  battery switch-over function is enabled in standard mode
                                                                                  //  battery low detection function is disabled
 ret=i2c_write(i2c1,writebuff,2,RTCADDR);                                          // write contents to register 3
 if(ret!=0)                                                                       // if there was a failure
      {                                                                            // give it  a little time
         if(!LOGGING)
             logmsg_add(2,"RTC SET REG 3 FAILED");                                      // write an error  message to msy 0 pool
       return;
      }
  
                                                             // go reset the AF and BF flags
  writebuff[0]=RTC_CONTROL_1_REG;                                                 // now point at CONTROL_REG_1
 ret=i2c_write_read(i2c1,RTCADDR,writebuff,1,readbuff,1);                         // and read the register
 if(ret!=0)                                                                       // if there was a failure
      {                                                                            
         if(!LOGGING)
            logmsg_add(2,"RTC GET REG 1 FAILED");                                     // write an error  message to msy 0 pool
       return;                                                                    // and return
      }
 test=readbuff[0]|0X02;                                                           // set the the AIE flag
  writebuff[0]=RTC_CONTROL_1_REG;                                                  // set up control register 3
 writebuff[1]=test;                                                               // ocntents are
                                                                                  //  battery switch-over function is enabled in standard mode
                                                                                  //  battery low detection function is disabled
 ret=i2c_write(i2c1,writebuff,2,RTCADDR);                                          // write contents to register 3
 if(ret!=0)                                                                       // if there was a failure
      {                                                                            // give it  a little time
         if(!LOGGING)
           logmsg_add(0,"RTC SET REG 1 FAILED");                                      // write an error  message to msy 0 pool
       return;
      }


 calc_reboot();                                                                   // the date has been set succesfully -- go calulate
                                                                                  // the reboot time
}
struct RTCDATE get_rtc()
{struct RTCDATE date,newdate;
struct EEPROMMSGS anchor;
struct LOGMSG *msg;
uint32_t posixnow,posixboot;
int32_t posixdiff;
uint8_t buffer[50];
 uint8_t writeaddr, readbuff[7],writebuff[30],buff[30];
 int ret,i;

 memset(&date,0,sizeof(date));                                                  // testinf onky - for nfc board that doesn't have a RTC
  date.second=0;
 writeaddr=RTC_SECONDS_REG;                                                      // point to tbe seconds register
 memset(readbuff,0,sizeof(readbuff));                                             // clear the read buffer
 ret=i2c_write_read(i2c1,RTCADDR,&writeaddr,1,readbuff,7);                        // and read the date and time
 date.second=from_bcd((readbuff[0]&0X7F));                                        // convert back to a reasonable fomat note that the seconds have a high order bit 
 date.minute=from_bcd(readbuff[1]);
 date.hour=from_bcd(readbuff[2]);
 date.day=from_bcd(readbuff[3]);
 date.month=from_bcd(readbuff[5]);
 date.year=from_bcd(readbuff[6])+2000;
 date.weekday=from_bcd(readbuff[4]);
 i=strncmp("ZB00000001",PARMS.lockid,10);                                         // if this is ZB00000001 - a lockbox under test
 if(i==0)
        return date;                                                              // we're done - send the date backto the caller
 return date;
 posixnow=dt_to_posix(date);                                                      // convert the date we just retrieved to a posix date
 RTCRESETCODE=0;                                                                  // set the reset code to
 posixboot=PARMS.LASTBOOT;                                                        // get the lasyt boot date
 if(posixboot==0)                                                                 // if there was no lastboot
      return date;                                                                // we can't do the date test - jut return
 posixdiff=posixnow-posixboot;                                                    // posix diff is the elapsed time between the last reboot and right now in seconds)
 if((posixdiff<31536000)&&(posixdiff>-1))                                         // if it's less than a year  (31,536,000 seconds) and positive
     return date;                                                                 //     we're good to go
 
 if(PARMS.USEAWS!='Y')                                                            // if we're not using AWS
 {                                                       
     for(i=0;i<5;i++)                                                             // all we can do is beep for a little while
     {                                                                            //     so we send five beeps
                                           
          k_sleep(K_MSEC(1000));
     }
     if(!LOGGING)                                                                 // if we're not already logging an event (which we probably are)
     {
        sprintf(buffer,"RTC of BY %d SECONDS",posixdiff);                         // log the face in the events log
        logmsg_add(2,buffer);
     }
     PARMS.LASTBOOT=posixnow;                                                     // set the LASTBOOT to whatever we just got (so this doesn't happen again)     
     eeprom_update_parms();                                                       // and store the PARAMS back ont the EEPROM
     return date;                                                                 // and send the date back
 }
                                                                 //
 RTCRESETCODE=1;                                                                  // otherwisee - try to recover the date
 quectel_powerdown();                                                             // power down the quecttel chip unilaterally
                                                                                  //    if it's not powered up this is fine
 if(!QPOWEREDUP)                                                                  // if the quectel chip isn't powered up
      ret= quectel_powerup();                                                     // try to power it up
 if(!QPOWEREDUP)                                                                  // if the powerup failed
  {     
        PARMS.LASTBOOT=posixnow;                                                  // set the LASTBOOT to whatever we just got (so this doesn't happen again)     
        eeprom_update_parms();                                                    // and store the PARAMS back ont the EEPROM 
        if(!LOGGING) 
           {                                                                      // if wer're not already creating a log record
            logmsg_add(2,"RTC REVOVERY FAILURE 3");                               //      log the failure
            sys_reboot(SYS_REBOOT_COLD);                                          // and reboot
           }
        else RTCRESETCODE=3;                                                      // otherwise set up to log that fact
        return(date);                                                             // and send back the bad date
 }
  if(!QCONNECTED)                                                                 // if we're not connected to AWS
       qconnect();                                                                // try to connect
  
  if(!QCONNECTED)                                                                 // if we failed
  {
       PARMS.LASTBOOT=posixnow;                                                   // set the LASTBOOT to whatever we just got (so this doesn't happen again)     
       eeprom_update_parms();                                                     // and store the PARAMS back ont the EEPROM 
       if(!LOGGING)                                                               // if wer're not already creating a log record
       {
            logmsg_add(2,"RTC REVOVERY FAILURE 4");                               //      log the failure
            sys_reboot(SYS_REBOOT_COLD);                                          // and reboot
       }
        else RTCRESETCODE=4;                                                      // otherwise set up to log that fact
        quectel_powerdown();                                                      // powerdown the quectel chip
        QPOWEREDUP=false;
        return(date);                                                             // and send back the bad date
 }
 newdate=qget_date();                                                             // go get the date from the server
 if(newdate.year==0)                                                              // if the year retuned was 0
  {                                                                               // there was a failure getting the date
        PARMS.LASTBOOT=posixnow;                                                  // set the LASTBOOT to whatever we just got (so this doesn't happen again)     
        eeprom_update_parms();                                                    // and store the PARAMS back ont the EEPROM 
        if(!LOGGING)                                                              // if wer're not already creating a log record
        {
            logmsg_add(2,"RTC REVOVERY FAILURE 5");                               //      log the failure
            sys_reboot(SYS_REBOOT_COLD);                                          // and reboot
        }
        else RTCRESETCODE=5; 
        que_disconnect(); 
        quectel_powerdown();                                                      // powerdown the quectel chip
        QCONNECTED=false;                                                         // reset the QUECTEL logicals
        QPOWEREDUP=false;  
        return(date);                                                              // and send back the bad date
 }
 que_disconnect();                                                               /// - success - disconnect from the network
 quectel_powerdown();                                                            // powerdown the quectel chip
 QCONNECTED=false;                                                               // reset the QUECTEL logicals
 QPOWEREDUP=false;
 set_rtc(newdate);                                                               // reset the rtc
 if(!LOGGING)                                                                    // if wer're not already creating a log record
 {
     PARMS.LASTBOOT=posixnow;                                                    // set the LASTBOOT to whatever we just got (so this doesn't happen again)     
     eeprom_update_parms();                                                      // and store the PARAMS back ont the EEPROM 
     logmsg_add(2,"RTC RECOVEREVED FROM AWS");                                   //      log the failure
     sys_reboot(SYS_REBOOT_COLD);                                                // and reboot
 }
 else RTCRESETCODE=99; 
 return newdate;                                                                 // return the new date
}
